/* MAC 440 - EP 3 - "MUD Distribuido (TCB)"

   Nomes: Andr Gustavo Andrade         #3117621
          Julio Cesar Avero             #3188339
	  
   Ferramentas: JacORB1_4_beta2, jakarta-ant-1.4.1, j2sdk1.4.0
   
   Data Incio:            15/06
   Data Ultima Alterao:  04/07
*/
/* TODO:
 */
package MUD;

import java.util.*;
import java.io.*;

/** Classe utilizada pelo MUD para armazenar e recuperar dados. Possui mtodos
para insero, remoo e atualizao de tuplas. 
*/
public class Repositorio 
    implements Serializable
{
    /** Quantidade mxima de campos em uma tupla deste repositrio. */
    private int POS_MAX = 7;
    /** Tamanho mximo de cada campo da tupla. */
    private int TAM_CAMPO_MAX = 1024;
    /** O ltimo identificador utilizado. Ele comea em 1 e vai crescendo uma unidade
       para cada registro inserido 
     */
    private long ultimoId;
    /** Onde realmente os dados so armazenados, sendo que a chave  o identificador
        do registro
     */
    private Hashtable tabela;

    /** 
      Classe que representa um n de uma lista de identificadores de registros
      retornados de uma busca. 
    */
    private class NoLista {
	/** */
	public long id;
	/** */
	public NoLista proximo;
	
	/** 
	    @param id, identificador do registro.
	*/
	NoLista (long id) {
	    proximo = null;
	    this.id = id;
	}               
    }
    
    /** Constri um repositorio de dados.
     */	
    public Repositorio () {
	ultimoId = 1;
	tabela = new Hashtable();
    }
	
    /** Gera um identificador nico no repositrio.
	@return o identificador.
    */
    private synchronized long GerarId () {
	return ultimoId++;
    }
	
    /** Adiciona um registro (tupla) no repositorio
	@param tupla Uma string descrevendo o registro, sendo que cada 
	             campo est em uma linha diferente, ou seja, os campos 
		     esto separados por '\n's                  
	@return o identificador deste novo objeto inserido.
    */
    public synchronized long Inserir (String tupla) {
	String[] campo = new String[POS_MAX];
	char[] buffer = new char[TAM_CAMPO_MAX]; 
	long id = GerarId();
		
	campo[0] = Long.toString(id);
	int j = 0, n = 1;
	for (int i = 0; (i < tupla.length()) && (n != TAM_CAMPO_MAX); i++) {			
	    buffer[j] = tupla.charAt(i);
	    j++;			
	    if ((tupla.charAt(i) == '\n') && (n < 7)) {
		campo[n] = new String (buffer, 0, j-1);
		j = 0; 
		n++;
	    }
	}
		
	tabela.put (Long.toString(id), campo);
	return id;
    }
	
    /** Devolve o id do registro desejado.
	@param campo Valor que queremos encontrar.
	@param pos Determina qual o campo (em qual coluna) estamos procurando.
	@return <code>null</code> se nenhuma ocorrncia foi encontrada, um vetor
	de <code>long</code> caso contrrio, que contm os ids dos registros desejados.
    */	
    public synchronized long[] PegarId (String campo, int pos) {
	if (pos >= POS_MAX) return null;  
	NoLista lista = new NoLista(0);
	   
	int tamLista = 0;
	for (Enumeration e = tabela.keys (); e.hasMoreElements () ;) {       
	    long id = Long.parseLong ((String) e.nextElement ());
	    String [] tupla = (String []) tabela.get(Long.toString(id));
	    if (campo.compareTo(tupla[pos]) == 0) {
		NoLista novo = new NoLista (id);
		novo.proximo = lista;
		lista = novo;
		tamLista++;
	    }
	}
	   
	if (tamLista > 0) {
		long [] ids = new long[tamLista]; 
		for (int i = 0; i < tamLista; i++, lista = lista.proximo) ids[i] = lista.id;
		return ids;   
	}
	else return null;
    }

    /** Remove um registro do repositorio.
	@param id O identificador do item que se quer remover
	@return <code>true</code> se foi possvel a remoo, <code>false</code> 
	        caso contrrio.
    */
    public synchronized boolean Remover (long id) {
	if (tabela.containsKey (Long.toString(id))) {
	    tabela.remove (Long.toString(id));			
	    return true;
	}
	return false;			
    }

    /** Le um determinado campo, do registro desejado.
	@param id Identificador do registro desejado.
	@param pos A posio do campo a ser lido.
	@return o campo que se quer ler, podendo ser <b>null</b> se 
	        registro ou campo inexiste.
    */
    public synchronized String PegarCampo (long id, int pos) {
	if ((tabela.containsKey (Long.toString(id))) && (pos < POS_MAX)) {
	    String[] tupla = (String []) tabela.get (Long.toString(id));
	    return tupla[pos];
	}		
	return null;	
    }	
	
    /** Le um determinado campo do registro desejado.
	@param id Identificador do registro cujo campo se quer alterar.
	@param pos A posio do campo que se quer alterar.
	@param campo O novo valor do campo.
	@return <code>true</code> se o campo foi alterado com sucesso, <code>false</code> 
                caso contrrio.
    */
    public synchronized boolean AlterarCampo (long id, int pos, String campo) {
	if ((tabela.containsKey (Long.toString(id))) && (pos < POS_MAX)) {
	    String[] tupla = (String []) tabela.get (Long.toString(id));
	    tupla[pos] = campo;
	    return true;
	}
	return false;
    }		
}
